/*
 *  Copyright (C) 2000, 2001 Ximian, Inc.
 *
 *  Authors: Joe Shaw <joe@ximian.com>
 *           Michael Zucchi <notzed@ximian.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNOME_TRANSFER_H
#define _GNOME_TRANSFER_H

#include <gtk/gtk.h>
#include <time.h>
#include "url-util.h"

#define GNOME_TRANSFER(obj)         GTK_CHECK_CAST (obj, gnome_transfer_get_type (), GnomeTransfer)
#define GNOME_TRANSFER_CLASS(klass) GTK_CHECK_CLASS_CAST (klass, gnome_transfer_get_type (), GnomeTransferClass)
#define IS_GNOME_TRANSFER(obj)      GTK_CHECK_TYPE (obj, gnome_transfer_get_type ())

typedef struct _GnomeTransfer        GnomeTransfer;
typedef struct _GnomeTransferClass   GnomeTransferClass;
typedef struct _GnomeTransferPrivate GnomeTransferPrivate;

typedef enum _GnomeTransferStatus {
	GNOME_TRANSFER_IDLE,	/* nothing started */
	GNOME_TRANSFER_LOOKUP,	/* looking up host */
	GNOME_TRANSFER_CONNECT,	/* connecting */
	GNOME_TRANSFER_DATA,	/* downloading data */

	/* any of these may be returned by the file_done signal */
	GNOME_TRANSFER_COMPLETE, /* finished */
	GNOME_TRANSFER_FAILED,	/* failure */
	GNOME_TRANSFER_ABORTED,	/* programmed failure */
	GNOME_TRANSFER_PAUSED,  /* transfer is paused */
} GnomeTransferStatus;

typedef enum _GnomeTransferType {
	GNOME_TRANSFER_TYPE_NONE,
	GNOME_TRANSFER_TYPE_HEADERS,
	GNOME_TRANSFER_TYPE_DATA
} GnomeTransferType;

struct _GnomeTransfer {
	GtkObject parent;

        GnomeTransferPrivate *priv;
	
	/* The type of transfer (headers only or data too) */
	GnomeTransferType type;

	/* The status of the current transfer */
	GnomeTransferStatus status;

	/* URL to connect to */
	Gurl *url;

	/* Reason for error. FIXME: move into private? */
	char *reason;

        /* Last modified time */
        time_t last_modified;

	/* Table of unparsed headers */
	GHashTable *headers;
};

struct _GnomeTransferClass {
	GtkObjectClass parent_class;

	void (*file_info)(GnomeTransfer *, char *name, int totalsize);
	void (*file_data)(GnomeTransfer *, char *buffer, int size);
	void (*file_done)(GnomeTransfer *, GnomeTransferStatus status);
	void (*file_paused)(GnomeTransfer *, int length);

	/* authentication required, should call _proxy_password(), and _retry() */
	void (*auth_required)(GnomeTransfer *, GnomeTransfer, char *domain);
};

guint		gnome_transfer_get_type	(void);
GnomeTransfer      *gnome_transfer_new	(void);

/* returns percent transferrred */
int gnome_transfer_percent(GnomeTransfer *t);
/* bytes so far */
int gnome_transfer_bytes(GnomeTransfer *t);
/* bytes total, -1 if not known */
int gnome_transfer_total(GnomeTransfer *t);

/* get reason for failure, if there is one */
char *gnome_transfer_reason(GnomeTransfer *t);

/* start a transfer */
int gnome_transfer_begin(GnomeTransfer *t, char *uri);
/* start a transfer, offset by an amount */
int gnome_transfer_begin_offset(GnomeTransfer *t, char *uri, int offset);
/* retry the last transfer */
int gnome_transfer_retry(GnomeTransfer *t);
/* abort the current transfer (not really working yet) */
void gnome_transfer_abort(GnomeTransfer *t);
/* only download headers instead of a whole transfer */
int gnome_transfer_headers(GnomeTransfer *t, char *uri);

/* pause/resume a transfer, resume restarts from where it was paused, but reconnects */
void gnome_transfer_pause(GnomeTransfer *t);
void gnome_transfer_resume(GnomeTransfer *t);

/* find out if it is complete */
GnomeTransferStatus gnome_transfer_status(GnomeTransfer *t);

/* set a proxy for transfer */
int gnome_transfer_proxy(GnomeTransfer *t, char *uri);

/* set basic auth info for proxy, if required */
void gnome_transfer_proxy_password(GnomeTransfer *t, char *name, char *pass);

/* verify that a file exists on the server side */
gboolean gnome_transfer_verify(char *url);

void gnome_transfer_emit_info(GnomeTransfer *t, char *name, int size);
void gnome_transfer_emit_data(GnomeTransfer *t, char *buf, int size);
void gnome_transfer_emit_done(GnomeTransfer *t, GnomeTransferStatus status);

#endif /* ! _GNOME_TRANSFER_H */
