/* X-Chat
 * Copyright (C) 1998 Peter Zelezny.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "../common/xchat.h"
#include "simple.h"
#include "xtext.h"

extern struct xchatprefs prefs;

static void timecpy (char *buf)
{
	time_t timval = time (0);
	char *tim = ctime (&timval) + 10;
	memcpy (buf, tim, 9);
	buf[0] = '[';
	buf[9] = ']';
	buf[10] = ' ';
}

static void
PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
{
	char *tab, *new_text;
	int leftlen;

	if (len == -1)
		len = strlen (text);

	if (len == 0)
		len = 1;

	if (!indent) {
		if (prefs.timestamp) {
			new_text = malloc (len + 12);
			timecpy (new_text);
			memcpy (new_text + 11, text, len);
			gtk_xtext_append (GTK_XTEXT (textwidget), new_text,
					  len + 11);
			free (new_text);
		} else
			gtk_xtext_append (GTK_XTEXT (textwidget), text, len);
		return;
	}

	tab = strchr (text, '\t');
	if (tab && (unsigned long) tab < (unsigned long) (text + len)) {
		leftlen = (unsigned long) tab - (unsigned long) text;
		gtk_xtext_append_indent (GTK_XTEXT (textwidget),
					 text, leftlen, tab + 1,
					 len - (leftlen + 1));
	} else
		gtk_xtext_append_indent (GTK_XTEXT (textwidget), 0, 0, text,
					 len);
}

void PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent)
{
	char *cr;

	if (!prefs.filterbeep && strchr (text, 7))
		gdk_beep ();

	cr = strchr (text, '\n');
	if (cr) {
		while (1) {
			PrintTextLine (textwidget, text,
				       (unsigned long) cr -
				       (unsigned long) text, indent);
			text = cr + 1;
			if (*text == 0)
				break;
			cr = strchr (text, '\n');
			if (!cr) {
				PrintTextLine (textwidget, text, -1, indent);
				break;
			}
		}
	} else {
		PrintTextLine (textwidget, text, -1, indent);
	}
}
